#ifndef __BOARD_H__
#define __BOARD_H__


#ifdef AT91SAM7S256
    #include "include/at91sam7s256.h"
    #include "include/lib_at91sam7s256.h"
#elif AT91SAM7X256
    #include "include/at91sam7x256.h"
    #undef __inline
    #define __inline static inline
    #include "include/lib_at91sam7x256.h"
#elif AT91SAM7S64
    #include "include/at91sam7s64.h"
    #undef __inline
    #define __inline static inline
    #include "include/lib_at91sam7s64.h"
#elif AT91SAM7XC128
	//use header for SAM7XC
	#include "include/at91sam7xc128.h"
	//use lib_ for SAM7X
    #undef __inline
    #define __inline static inline
    #include "include/lib_at91sam7x128.h"
#elif AT91SAM7XC256
	//use header for SAM7XC
	#include "include/at91sam7xc256.h"
	//use lib_ for SAM7X
    #undef __inline
    #define __inline static inline
    #include "include/lib_at91sam7x256.h"
#else
    #error SUBMDL, no match!
#endif

//common includes
#include "delay.h"

//global defs
#define EXT_OC          18432000   // Exetrnal ocilator MAINCK
#define MCK             48054857   // MCK (PLLRC div by 2)

#define OK				1
#define FAIL			0

//=================================================================================================
//								Switches
//=================================================================================================

#define KEYBOARD_PIO	AT91C_BASE_PIOA
#define KEYBOARD_PIO_ID	AT91C_ID_PIOA
#define SW1_MASK		AT91C_PIO_PA7
#define SW2_MASK		AT91C_PIO_PA6
#define SW3_MASK		AT91C_PIO_PA5
#define SW4_MASK		AT91C_PIO_PA2

#define SW1_PRESSED		( (KEYBOARD_PIO->PIO_PDSR & SW1_MASK) == 0 )
#define SW2_PRESSED		( (KEYBOARD_PIO->PIO_PDSR & SW2_MASK) == 0 )
#define SW3_PRESSED		( (KEYBOARD_PIO->PIO_PDSR & SW3_MASK) == 0 )
#define SW4_PRESSED		( (KEYBOARD_PIO->PIO_PDSR & SW4_MASK) == 0 )




//=================================================================================================
//								LEDs
//=================================================================================================

//LED Red
#define LED_R_PIO		AT91C_BASE_PIOB
#define LED_R_MASK		AT91C_PIO_PB23
//LED Yellow
#define LED_Y_PIO		AT91C_BASE_PIOB
#define LED_Y_MASK		AT91C_PIO_PB24
//LED Green
#define LED_G_PIO		AT91C_BASE_PIOB
#define LED_G_MASK		AT91C_PIO_PB25
//LED Blue
#define LED_B_PIO		AT91C_BASE_PIOB
#define LED_B_MASK		AT91C_PIO_PB26

//LEDs interface defs -- to be used in the program
#define LED_R_INIT		{LED_R_PIO->PIO_PER = LED_R_MASK; LED_R_PIO->PIO_OER = LED_R_MASK; LED_R_PIO->PIO_OWER = LED_R_MASK;}
#define LED_R_ON		LED_R_PIO->PIO_SODR = LED_R_MASK
#define LED_R_OFF		LED_R_PIO->PIO_CODR = LED_R_MASK
#define LED_R_TOGGLE	LED_R_PIO->PIO_ODSR ^= LED_R_MASK

#define LED_Y_INIT		{LED_Y_PIO->PIO_PER = LED_Y_MASK; LED_Y_PIO->PIO_OER = LED_Y_MASK; LED_Y_PIO->PIO_OWER = LED_Y_MASK;}
#define LED_Y_ON		LED_Y_PIO->PIO_SODR = LED_Y_MASK
#define LED_Y_OFF		LED_Y_PIO->PIO_CODR = LED_Y_MASK
#define LED_Y_TOGGLE	LED_Y_PIO->PIO_ODSR ^= LED_Y_MASK

#define LED_B_INIT		{LED_B_PIO->PIO_PER = LED_B_MASK; LED_B_PIO->PIO_OER = LED_B_MASK; LED_B_PIO->PIO_OWER = LED_B_MASK;}
#define LED_B_ON		LED_B_PIO->PIO_SODR = LED_B_MASK
#define LED_B_OFF		LED_B_PIO->PIO_CODR = LED_B_MASK
#define LED_B_TOGGLE	LED_B_PIO->PIO_ODSR ^= LED_B_MASK

#define LED_G_INIT		{LED_G_PIO->PIO_PER = LED_G_MASK; LED_G_PIO->PIO_OER = LED_G_MASK; LED_G_PIO->PIO_OWER = LED_G_MASK;}
#define LED_G_ON		LED_G_PIO->PIO_SODR = LED_G_MASK
#define LED_G_OFF		LED_G_PIO->PIO_CODR = LED_G_MASK
#define LED_G_TOGGLE	LED_G_PIO->PIO_ODSR ^= LED_G_MASK




#endif
