#include "common.h"
#include "lcd_2x16.h"
#include "adc.h"

#define ADC_BUFFER_SIZE		20	//in 16-bit words and doubled!

//buffers for samples
static volatile uint16_t adcBuffer[2][ADC_BUFFER_SIZE];
static volatile uint8_t activeBuffer;

//prototype for ADC DMA ISR function
void adcDmaBufferSwap(void);


void init(void)
{
	// enable peripheral clock for PIO controllers
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA);
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOB);
	// enable reset-key on board 
	AT91C_BASE_RSTC->RSTC_RMR = (0xA5000000 | AT91C_RSTC_URSTEN);
	
	debug_init_default();
	debug_msg("debug channel initialized");
	
	//initialize LEDs
	LED_R_INIT;
	LED_Y_INIT;
	LED_G_INIT;
	LED_B_INIT;
	
	lcdInit();
	lcdLocate(0,0);
	lcdText("SW1 to start");
	lcdLocate(1,0);
	lcdText("Data on DBGU");
}




int main( void )
{
	init();
	
	//define channel to sample
	const uint8_t ADC_VR_CHANNEL = 4;
	const uint32_t ADC_DEFAULT_SAMPLING_RATE = 10;	//samples per second
	uint8_t running = 0;
	
	for(;;)
	{
		if((SW1_PRESSED)&&(running==0))
		{
			//message for the user
			lcdLocate(0,0);
			lcdText("SW2 to stop ");
			
			//set flag
			running = 1;
			
			//start DMA transfers from ADC
			adcDmaStart(ADC_VR_CHANNEL, ADC_DEFAULT_SAMPLING_RATE, adcDmaBufferSwap);
			
			LED_G_ON;
			
			//debounce pushbutton
			while(SW1_PRESSED);
			delay_ms(100);
		}
		
		if(SW2_PRESSED)
		{
			//message for the user
			lcdLocate(0,0);
			lcdText("SW1 to start");
			
			//clear flag
			running = 0;
			
			//stop DMA transfers from ADC
			adcDmaStop();
			
			LED_G_OFF;
			
			//debounce pushbutton
			while(SW1_PRESSED);
			delay_ms(100);
		}
		
		LED_B_TOGGLE;
		delay_ms(50);
	}
	return(0);
	
}



//this function is called from adc.c module on ADC's RXBUFF and ENDRX flags
void adcDmaBufferSwap(void)
{

	debug_msg("ISR");
	
	//copy adc status to a variable
	static uint32_t status;
	status = ADC->ADC_SR;
	
	//both buffers are not initialized or full
	if(status & AT91C_ADC_RXBUFF)
	{
		debug_msg("DMA Init");
		activeBuffer = 0;
		ADC->ADC_RPR = (uint32_t) &adcBuffer[0][0];
		ADC->ADC_RNPR = (uint32_t) &adcBuffer[1][0];
		ADC->ADC_RCR = ADC_BUFFER_SIZE;
		ADC->ADC_RNCR = ADC_BUFFER_SIZE;
		ADC->ADC_PTCR = AT91C_PDC_RXTEN;
		return;
	}
	
	//buffer filling finished
	if(status & AT91C_ADC_ENDRX)
	{
		static uint8_t bufferToDisplay;
		if(activeBuffer)
		{
			debug_msg("RXBUFF 0");
			activeBuffer = 0;
			bufferToDisplay = 1;
		}
		else
		{
			debug_msg("RXBUFF 1");
			activeBuffer = 1;
			bufferToDisplay = 0;
		}
		//prepare "next" buffer
		ADC->ADC_RNPR = (uint32_t) &adcBuffer[activeBuffer][0];
		ADC->ADC_RNCR = ADC_BUFFER_SIZE;
		
		//this is a good place to copy/process/display data in buffer that has just been rxed
		if(activeBuffer)
			debug_msg("Buffer 0 ready: ");
		else
			debug_msg("Buffer 1 ready: ");
		
		debug_array(&adcBuffer[bufferToDisplay][0],ADC_BUFFER_SIZE);
	}
	
}


