#include "common.h"
#include "pwm_dac.h"

#define PWM_CHANNEL		3
#define PWM				AT91C_BASE_PWMC
#define PWM_ID			AT91C_ID_PWMC
#define PWM_PIO			AT91C_BASE_PIOB
#define PWM_PIN_MASK	AT91C_PB30_PWM3
#define PWM_PERIPH		1		//0 for periph A
#define PWM_MAX_COUNT	255
#define PWM_DAC_TIMER			AT91C_BASE_TC1
#define PWM_DAC_TIMER_ID		AT91C_ID_TC1


//Clock selection for Timer
#define TC_CLKS					0x7
#define TC_CLKS_MCK2			0x0
#define TC_CLKS_MCK8			0x1
#define TC_CLKS_MCK32			0x2
#define TC_CLKS_MCK128			0x3
#define TC_CLKS_MCK1024			0x4
#define SELECTED_TC_CLKS		TC_CLKS_MCK32
#define TC_PRESCALER			32


void pwmdacTimerInit(uint32_t sampleRate);


void pwmdacInit(uint32_t sampleRate, void(*sampleIsrFunc)())
{
    //enable PWMC clock
    AT91C_BASE_PMC->PMC_PCER = 1<<PWM_ID;
	
	//set initial value of duty cycle
	PWM->PWMC_CH[PWM_CHANNEL].PWMC_CDTYR = PWM_MAX_COUNT/2;
	
	//set PWM period (max count)
	PWM->PWMC_CH[PWM_CHANNEL].PWMC_CPRDR = PWM_MAX_COUNT;
	
	//set Max Frequency (MCK)
	PWM->PWMC_CH[PWM_CHANNEL].PWMC_CMR = 0;
	
	//disable PIO controller on PWM pin
	PWM_PIO->PIO_PDR = PWM_PIN_MASK;
	
	//select peripheral
	#if PWM_PERIPH
		PWM_PIO->PIO_BSR = PWM_PIN_MASK;
	#else
		PWM_PIO->PIO_ASR = PWM_PIN_MASK;
	#endif
	
	PWM->PWMC_ENA = (1<<PWM_CHANNEL);
	
	pwmdacTimerInit(sampleRate);
	
	const uint8_t DAC_PWM_PRIORITY = 1;
	
	//prepare AIC for Timer interrupts
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, PWM_DAC_TIMER_ID, DAC_PWM_PRIORITY, AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL , sampleIsrFunc);
	
	//enable interrupts from Timer on AIC
	AT91F_AIC_EnableIt( AT91C_BASE_AIC, PWM_DAC_TIMER_ID );
	
	//enable interrupt generation on CPCS bit
	PWM_DAC_TIMER->TC_IER  = AT91C_TC_CPCS;
	
}

inline void pwmdacSet(uint8_t value)
{
	//PWM->PWMC_CH[PWM_CHANNEL].PWMC_CDTYR = value;
	PWM->PWMC_CH[PWM_CHANNEL].PWMC_CUPDR = value;
}


inline uint32_t pwmdacGetTimerStatus(void)
{
	return(PWM_DAC_TIMER->TC_SR);
}


void pwmdacTimerInit(uint32_t sampleRate)
{
	//enable the clock of the PWM_DAC_TIMER
	AT91C_BASE_PMC->PMC_PCER = 1<<PWM_DAC_TIMER_ID;
	
	//disable timer's clock
	PWM_DAC_TIMER->TC_CCR = AT91C_TC_CLKDIS;
	
	//disable timer's interrupts
	PWM_DAC_TIMER->TC_IDR = 0xFFFFFFFF ;
	
	//read status register to clear its flags
	uint32_t dummy;
	dummy = AT91C_BASE_TC0->TC_SR;
	dummy = dummy;
	
	//set capture mode register of the timer
	PWM_DAC_TIMER->TC_CMR = AT91C_TC_WAVE  //bit WAVE=1 => Waveform Mode
		|SELECTED_TC_CLKS   //Timer clock freq.
		|AT91C_TC_WAVESEL_UP_AUTO;  //Waveform Selection: UP mode with automatic trigger on RC compare (10)
	
	//write full-count value into RC
	PWM_DAC_TIMER->TC_RC = MCK/TC_PRESCALER/sampleRate;
	
    //enable timer's clock
	PWM_DAC_TIMER->TC_CCR = AT91C_TC_CLKEN;
	
    //start timer
    PWM_DAC_TIMER->TC_CCR = AT91C_TC_SWTRG ;
	
}



