#include "common.h"
#include "msc_bot.h"
#include "msc_usb.h"
#include "msc_scsi.h"
#include "msc_if.h"
#include "msc_ui.h"



//internal defs
/* USB standard request code */
#define USB_STD_GET_STATUS_ZERO           0x0080
#define USB_STD_GET_STATUS_INTERFACE      0x0081
#define USB_STD_GET_STATUS_ENDPOINT       0x0082

#define USB_STD_CLEAR_FEATURE_ZERO        0x0100
#define USB_STD_CLEAR_FEATURE_INTERFACE   0x0101
#define USB_STD_CLEAR_FEATURE_ENDPOINT    0x0102

#define USB_STD_SET_FEATURE_ZERO          0x0300
#define USB_STD_SET_FEATURE_INTERFACE     0x0301
#define USB_STD_SET_FEATURE_ENDPOINT      0x0302

#define USB_STD_SET_ADDRESS               0x0500
#define USB_STD_GET_DESCRIPTOR            0x0680
#define USB_STD_SET_DESCRIPTOR            0x0700
#define USB_STD_GET_CONFIGURATION         0x0880
#define USB_STD_SET_CONFIGURATION         0x0900
#define USB_STD_GET_INTERFACE             0x0A81
#define USB_STD_SET_INTERFACE             0x0B01
#define USB_STD_SYNCH_FRAME               0x0C82

/* Typy deskryptorw */
#define USB_DSCT_DEVICE_DESCRIPTOR          0x100
#define USB_DSCT_CONFIGURATION_DESCRIPTOR   0x200
#define USB_DSCT_STRING_DESCRIPTOR          0x300
#define USB_DSCT_INTERFACE_DESCRIPTOR       0x400
#define USB_DSCT_ENDPOINT_DESCRIPTOR        0x500
#define USB_DSCT_DEVICE_QUALIFIER_DESCRIPTOR 0x600

#define USB_MSC_MASS_STORAGE_RESET          0xFF21
#define USB_MSC_GET_MAX_LUN                 0xFEA1

/* Indeksy dla deskryptorw tekstowych */
#define USB_STRDSC_LANGUAGE                 0
#define USB_STRDSC_MANUFACTURER             1
#define USB_STRDSC_PRODUCT                  2
#define USB_STRDSC_SERIAL_NUMBER            3

#define TO_HOST                             1
#define FROM_HOST                           0


#include "msc_desc.h"

#define MIN(a, b) (((a) < (b)) ? (a) : (b))


#define USB_STATUS_REMOTE_WAKEUP        0x0200
#define USB_STATUS_SELF_POWERED         0x0100
#define USB_STATUS                      0

#define BULK_IN_ENDPOINT_SIZE               0x40
#define BULK_OUT_ENDPOINT_SIZE              0x40


#define BULK_IN_ENDPOINT                    1
#define BULK_OUT_ENDPOINT                   2






void USB_SendData(uint8_t *pData, uint32_t length);
void USB_SendZlp(void);



static uint16_t USB_CurrentConfiguration;

volatile int USB_EP2_CurrentBank;

uint8_t USB_EP2_Buffer[0x40];

volatile uint8_t mscMaxLUN;


void mscEnumerate(void)
{
	
    AT91_REG Status;
	uint8_t bmRequestType, bRequest;
	uint16_t wValue, wIndex, wLength, wStatus;
	
    //status read
    Status = (AT91C_BASE_UDP)->UDP_ISR;
	
    if (Status & AT91C_UDP_ENDBUSRES)
    {
        //reset request
		(AT91C_BASE_UDP)->UDP_ICR = AT91C_UDP_ENDBUSRES;
		//endpoint reset
		(AT91C_BASE_UDP)->UDP_RSTEP  = (unsigned int)-1;
		(AT91C_BASE_UDP)->UDP_RSTEP  = 0;
		//address mode
		(AT91C_BASE_UDP)->UDP_FADDR = AT91C_UDP_FEN;
		//endpoint 0 config
		(AT91C_BASE_UDP)->UDP_CSR[0] = (AT91C_UDP_EPEDS | AT91C_UDP_EPTYPE_CTRL);
	}
    else
    {
	    if (Status & AT91C_UDP_EPINT0)	//new EP0 event
		{
		    (AT91C_BASE_UDP)->UDP_ICR = AT91C_UDP_EPINT0;
            //setup?
		    if ((AT91C_BASE_UDP)->UDP_CSR[0] & AT91C_UDP_RXSETUP)
            {
                // Nadszed pakiet SETUP, analizujemy zawarto
	            bmRequestType = (AT91C_BASE_UDP)->UDP_FDR[0];
	            bRequest      = (AT91C_BASE_UDP)->UDP_FDR[0];
	            wValue        = ((AT91C_BASE_UDP)->UDP_FDR[0] & 0xFF);
	            wValue       |= ((AT91C_BASE_UDP)->UDP_FDR[0] << 8);
	            wIndex        = ((AT91C_BASE_UDP)->UDP_FDR[0] & 0xFF);
	            wIndex       |= ((AT91C_BASE_UDP)->UDP_FDR[0] << 8);
	            wLength       = ((AT91C_BASE_UDP)->UDP_FDR[0] & 0xFF);
	            wLength      |= ((AT91C_BASE_UDP)->UDP_FDR[0] << 8);
                
                
                
	            if (bmRequestType & 0x80) {
                    //IN direction
		            (AT91C_BASE_UDP)->UDP_CSR[0] |= AT91C_UDP_DIR;
                    while ( !((AT91C_BASE_UDP)->UDP_CSR[0] & AT91C_UDP_DIR) ) {
                        ;
                    }
	            }
                //clear setup flag
	            (AT91C_BASE_UDP)->UDP_CSR[0] &= ~AT91C_UDP_RXSETUP;
                while ( ((AT91C_BASE_UDP)->UDP_CSR[0]  & AT91C_UDP_RXSETUP)  ) {
                    ;
                }
                
	            switch ((bRequest << 8) | bmRequestType)
                {
                    case USB_STD_GET_DESCRIPTOR:
                        //descriptor request
                        switch (wValue & 0xff00)
                        {
                        case USB_DSCT_DEVICE_DESCRIPTOR:
                            //device descriptor
                            USB_SendData((uint8_t *) USB_DeviceDescriptor, MIN(sizeof(USB_DeviceDescriptor), wLength));
                            
                            break;
                        case USB_DSCT_CONFIGURATION_DESCRIPTOR:
                            //configuration descriptor
                            USB_SendData((uint8_t*)USB_ConfigDescriptor, MIN(sizeof(USB_ConfigDescriptor), wLength));
                            break;
                        case USB_DSCT_STRING_DESCRIPTOR:
                            //string descriptor
                            switch (wValue & 0x00ff)
                            {
                                case USB_STRDSC_LANGUAGE:
                                    USB_SendData((uint8_t*)USB_LanguageStringDescriptor, MIN(sizeof(USB_ManufacturerStringDescriptor), wLength));
                                    break;
                                case USB_STRDSC_MANUFACTURER:
                                    USB_SendData((uint8_t*)USB_ManufacturerStringDescriptor, MIN(sizeof(USB_ManufacturerStringDescriptor), wLength));
                                    break;
                                case USB_STRDSC_PRODUCT:
                                    USB_SendData((uint8_t*)USB_ProductStringDescriptor, MIN(sizeof(USB_ProductStringDescriptor), wLength));
                                    break;
                                case USB_STRDSC_SERIAL_NUMBER:
                                    USB_SendData((uint8_t*)USB_SerialNumberStringDescriptor, MIN(sizeof(USB_SerialNumberStringDescriptor), wLength));
                                    break;
                                default:
                                    //(bRequest << 8) | bmRequestType
                                    break;
                            }
                            break;
                            
                        case USB_DSCT_DEVICE_QUALIFIER_DESCRIPTOR:
                            USB_SendStall(0);
                            break;
                        default:
                            break;
                        } // switch (wValue & 0xff00)
                    break; //case USB_STD_GET_DESCRIPTOR:
	            case USB_STD_SET_ADDRESS:
		            USB_SendZlp();	//confirm
		            (AT91C_BASE_UDP)->UDP_FADDR = (AT91C_UDP_FEN | wValue);
		            (AT91C_BASE_UDP)->UDP_GLBSTATE  = (wValue) ? AT91C_UDP_FADDEN : 0;
                    
		            break;
                case USB_STD_SET_CONFIGURATION:
		            USB_CurrentConfiguration = wValue;
		            USB_SendZlp();	//confirm
					//set fieldz
                    AT91C_BASE_UDP->UDP_GLBSTATE = AT91C_UDP_CONFG;
                    AT91C_BASE_UDP->UDP_CSR[1] = (AT91C_UDP_EPEDS | AT91C_UDP_EPTYPE_BULK_IN);
                    AT91C_BASE_UDP->UDP_CSR[2] = (AT91C_UDP_EPEDS | AT91C_UDP_EPTYPE_BULK_OUT);
                    
		            break;
	            case USB_STD_GET_CONFIGURATION:
		            USB_SendData((uint8_t *) &(USB_CurrentConfiguration), sizeof(USB_CurrentConfiguration));
		            break;
                //rabw:
                /*case USB_STD_GET_INTERFACE:
		            USB_SendData((uint8_t *) &(USB_CurrentConfiguration), sizeof(USB_CurrentConfiguration));
                    break;*/
	            case USB_STD_GET_STATUS_ZERO:
		            wStatus = USB_STATUS;
		            USB_SendData((uint8_t *) &wStatus, sizeof(wStatus));
		            break;
	            case USB_STD_GET_STATUS_INTERFACE:
		            wStatus = 0;	//magic zero
		            USB_SendData((uint8_t *) &wStatus, sizeof(wStatus));
		            break;
	            case USB_STD_GET_STATUS_ENDPOINT:
		            wStatus = 0;
		            wIndex &= 0x0F;
		            if (((AT91C_BASE_UDP)->UDP_GLBSTATE & AT91C_UDP_CONFG) && (wIndex <= 3)) {
			            wStatus = ((AT91C_BASE_UDP)->UDP_CSR[wIndex] & AT91C_UDP_EPEDS) ? 0 : 1;
			            USB_SendData((uint8_t *) &wStatus, sizeof(wStatus));
		            }
		            else if (((AT91C_BASE_UDP)->UDP_GLBSTATE & AT91C_UDP_FADDEN) && (wIndex == 0)) {
			            wStatus = ((AT91C_BASE_UDP)->UDP_CSR[wIndex] & AT91C_UDP_EPEDS) ? 0 : 1;
			            USB_SendData((uint8_t *) &wStatus, sizeof(wStatus));
		            } else {
			            USB_SendStall(0);
                    }
                    
                    
		            break;
	            case USB_STD_SET_FEATURE_ZERO:
		            USB_SendStall(0);
                    
	                break;
	            case USB_STD_SET_FEATURE_INTERFACE:
		            USB_SendZlp();
                    
		            break;
	            case USB_STD_SET_FEATURE_ENDPOINT:
		            wIndex &= 0x0F;
		            if ((wValue == 0) && wIndex && (wIndex <= 3)) {
			            (AT91C_BASE_UDP)->UDP_CSR[wIndex] = 0;
			            USB_SendZlp();
                    } else {
			            USB_SendStall(0);
                    }
                    
		            break;
	            case USB_STD_CLEAR_FEATURE_ZERO:
		            USB_SendStall(0);
                    
	                break;
	            case USB_STD_CLEAR_FEATURE_INTERFACE:
		            USB_SendZlp();
                    
		            break;
	            case USB_STD_CLEAR_FEATURE_ENDPOINT:
		            wIndex &= 0x0F;
		            if ((wValue == 0) && wIndex && (wIndex <= 3)) {
			            if (wIndex == 1)
				            (AT91C_BASE_UDP)->UDP_CSR[1] = (AT91C_UDP_EPEDS | AT91C_UDP_EPTYPE_BULK_IN);
			            else if (wIndex == 2)
				            (AT91C_BASE_UDP)->UDP_CSR[2] = (AT91C_UDP_EPEDS | AT91C_UDP_EPTYPE_BULK_OUT);
			            USB_SendZlp();
                    }
                    else
                    {
			            USB_SendStall(0);
                    }
                    
		            break;
                default:
					//class-specific
					switch ((bRequest << 8) | bmRequestType)
					{
						case USB_MSC_GET_MAX_LUN:
							USB_SendData((uint8_t *)&mscMaxLUN, 1);
							break;
						default:
							//todo: other Class-specific requests processing
							//works without it too ;)
							break;
					}//switch ((bRequest << 8) | bmRequestType)
                    break;
                } // switch ((bRequest << 8) | bmRequestType)              
            } // if ((AT91C_BASE_UDP)->UDP_CSR[0] & AT91C_UDP_RXSETUP) 
        } // if (Status & AT91C_UDP_EPINT0)
        
        
        //auto data-out transfer service
	    if (Status & AT91C_UDP_EPINT2)
        {
            //data in EP2, yeah!
		    (AT91C_BASE_UDP)->UDP_ICR = AT91C_UDP_EPINT2;
            if ((AT91C_BASE_UDP)->UDP_CSR[2] & USB_EP2_CurrentBank) {
                mscProcessPacketTransfer();
            }
        } // if (Status & AT91C_UDP_EPINT2)
    } // else
}











//*----------------------------------------------------------------------------
//* \fn    USB_SendZlp
//* \brief Send zero length packet through the control endpoint
//*----------------------------------------------------------------------------
void USB_SendZlp(void)
{
	(AT91C_BASE_UDP)->UDP_CSR[0] |= AT91C_UDP_TXPKTRDY;
	while ( !((AT91C_BASE_UDP)->UDP_CSR[0] & AT91C_UDP_TXCOMP) );
	(AT91C_BASE_UDP)->UDP_CSR[0] &= ~(AT91C_UDP_TXCOMP);
	while ((AT91C_BASE_UDP)->UDP_CSR[0] & AT91C_UDP_TXCOMP);
}

//*----------------------------------------------------------------------------
//* \fn    USB_SendStall
//* \brief Stall the control endpoint
//*----------------------------------------------------------------------------
void USB_SendStall(uint8_t ep)
{
	(AT91C_BASE_UDP)->UDP_CSR[ep] |= AT91C_UDP_FORCESTALL;
	while ( !((AT91C_BASE_UDP)->UDP_CSR[ep] & AT91C_UDP_ISOERROR) );
	(AT91C_BASE_UDP)->UDP_CSR[ep] &= ~(AT91C_UDP_FORCESTALL | AT91C_UDP_ISOERROR);
	while ((AT91C_BASE_UDP)->UDP_CSR[ep] & (AT91C_UDP_FORCESTALL | AT91C_UDP_ISOERROR));
}



//requires:
	//pData - pointer to data buffer
	//limit - number of bytes to be read in one shot
//returns:
	//returns number of bytes less than limit
uint16_t USB_DataOutLimit(uint8_t *pData, uint16_t limit)
{
	for(;;)
	{
	    if (AT91C_BASE_UDP->UDP_ISR & AT91C_UDP_EPINT2)
        {
		    (AT91C_BASE_UDP)->UDP_ICR = AT91C_UDP_EPINT2;
            if ((AT91C_BASE_UDP)->UDP_CSR[2] & USB_EP2_CurrentBank)
            {
				uint16_t dataLength = USB_DataOut(pData);
				if(dataLength<limit)
				{
					pData+=dataLength;
				}
				else
				{
					return(0);
				}
			}
		}
	}
	return(0);
}




//RETURNS:  data length,
//REQUIRES: pointer to data array to store incoming data
uint32_t USB_DataOut(uint8_t *pData)
{
    uint32_t packet_length;
    packet_length = (AT91C_BASE_UDP)->UDP_CSR[BULK_OUT_ENDPOINT] >> 16;
    
    uint32_t i = 0;
    for(i=0;i<packet_length;i++)
    {
        *pData=AT91C_BASE_UDP->UDP_FDR[BULK_OUT_ENDPOINT]; //write value
        pData++;                            //and increment pointer
    }
    (AT91C_BASE_UDP)->UDP_CSR[BULK_OUT_ENDPOINT] &= ~(USB_EP2_CurrentBank);
    if (USB_EP2_CurrentBank == AT91C_UDP_RX_DATA_BK0) {
        USB_EP2_CurrentBank = AT91C_UDP_RX_DATA_BK1;
    } else {
        USB_EP2_CurrentBank = AT91C_UDP_RX_DATA_BK0;
    }
    return(packet_length);
}



uint32_t USB_DataIn(uint8_t *pData, uint32_t length)
{
	uint32_t cpt = 0;
    AT91_REG isr;

	// Send the first packet
	cpt = MIN(length, BULK_IN_ENDPOINT_SIZE);
	length -= cpt;
    while (cpt--)
    {
        (AT91C_BASE_UDP)->UDP_FDR[BULK_IN_ENDPOINT] = *pData++;
    }
	(AT91C_BASE_UDP)->UDP_CSR[BULK_IN_ENDPOINT] |= AT91C_UDP_TXPKTRDY;

	while (length)
    {
		// Fill the second bank
		cpt = MIN(length, BULK_IN_ENDPOINT_SIZE);
		length -= cpt;
        while (cpt--)
		{
            (AT91C_BASE_UDP)->UDP_FDR[BULK_IN_ENDPOINT] = *pData++;
        }
		// Wait for the the first bank to be sent
        while ( !((AT91C_BASE_UDP)->UDP_CSR[BULK_IN_ENDPOINT] & AT91C_UDP_TXCOMP) )
        {
            isr = (AT91C_BASE_UDP)->UDP_ISR;/////////////////////////////////////////////////////////////////////
            if (isr & AT91C_UDP_ENDBUSRES)
            {
		        (AT91C_BASE_UDP)->UDP_ICR = AT91C_UDP_ENDBUSRES;
		        // reset all endpoints
		        (AT91C_BASE_UDP)->UDP_RSTEP  = (unsigned int)-1;
		        (AT91C_BASE_UDP)->UDP_RSTEP  = 0;
		        // Enable the function
		        (AT91C_BASE_UDP)->UDP_FADDR = AT91C_UDP_FEN;
		        // Configure endpoint 0
    	        (AT91C_BASE_UDP)->UDP_CSR[0] = (AT91C_UDP_EPEDS | AT91C_UDP_EPTYPE_CTRL);
      	    } else if (isr & AT91C_UDP_EPINT0) {
		        (AT91C_BASE_UDP)->UDP_ICR = AT91C_UDP_EPINT0;
	        }     
        }
		(AT91C_BASE_UDP)->UDP_CSR[BULK_IN_ENDPOINT] &= ~(AT91C_UDP_TXCOMP);
        while ((AT91C_BASE_UDP)->UDP_CSR[BULK_IN_ENDPOINT] & AT91C_UDP_TXCOMP)
        {
            ;
        }
		(AT91C_BASE_UDP)->UDP_CSR[BULK_IN_ENDPOINT] |= AT91C_UDP_TXPKTRDY;
	}
	// Wait for the end of transfer
    while ( !((AT91C_BASE_UDP)->UDP_CSR[BULK_IN_ENDPOINT] & AT91C_UDP_TXCOMP) )
    {
            isr = (AT91C_BASE_UDP)->UDP_ISR;/////////////////////////////////////////////////////////////////////
            if (isr & AT91C_UDP_ENDBUSRES)
            {
		        (AT91C_BASE_UDP)->UDP_ICR = AT91C_UDP_ENDBUSRES;
		        // reset all endpoints
		        (AT91C_BASE_UDP)->UDP_RSTEP  = (unsigned int)-1;
		        (AT91C_BASE_UDP)->UDP_RSTEP  = 0;
		        // Enable the function
		        (AT91C_BASE_UDP)->UDP_FADDR = AT91C_UDP_FEN;
		        // Configure endpoint 0
    	        (AT91C_BASE_UDP)->UDP_CSR[0] = (AT91C_UDP_EPEDS | AT91C_UDP_EPTYPE_CTRL);
      	    } else if (isr & AT91C_UDP_EPINT0)
            {
		        (AT91C_BASE_UDP)->UDP_ICR = AT91C_UDP_EPINT0;
	        } 
    }
	(AT91C_BASE_UDP)->UDP_CSR[BULK_IN_ENDPOINT] &= ~(AT91C_UDP_TXCOMP);
	while ((AT91C_BASE_UDP)->UDP_CSR[BULK_IN_ENDPOINT] & AT91C_UDP_TXCOMP);

	return length;

}




//*----------------------------------------------------------------------------
//* \fn    USB_SendData
//* \brief Send Data through the control endpoint
//*----------------------------------------------------------------------------
void USB_SendData(uint8_t *pData, uint32_t length)
{
   uint32_t cpt = 0;
   AT91_REG csr;
   uint8_t stay_in_loop=0;
   
   if((length % 8)==0) stay_in_loop = 1;
   
   do
   {
      cpt = MIN(length, 8);
      length -= cpt;
      
      while (cpt--)
      {
         (AT91C_BASE_UDP)->UDP_FDR[0] = *pData++;
      }
      
      if ((AT91C_BASE_UDP)->UDP_CSR[0] & AT91C_UDP_TXCOMP)
      {
         (AT91C_BASE_UDP)->UDP_CSR[0] &= ~(AT91C_UDP_TXCOMP);
         while ((AT91C_BASE_UDP)->UDP_CSR[0] & AT91C_UDP_TXCOMP);
      }
      
      (AT91C_BASE_UDP)->UDP_CSR[0] |= AT91C_UDP_TXPKTRDY;
      
      do
      {
         csr = (AT91C_BASE_UDP)->UDP_CSR[0];
         // Data IN stage has been stopped by a status OUT
         if (csr & AT91C_UDP_RX_DATA_BK0)
         {
            (AT91C_BASE_UDP)->UDP_CSR[0] &= ~(AT91C_UDP_RX_DATA_BK0);
            return;
         }
      } while ( !(csr & AT91C_UDP_TXCOMP) );
      
   }while(length||stay_in_loop);
   
   if ((AT91C_BASE_UDP)->UDP_CSR[0] & AT91C_UDP_TXCOMP)
   {
      (AT91C_BASE_UDP)->UDP_CSR[0] &= ~(AT91C_UDP_TXCOMP);
      while ((AT91C_BASE_UDP)->UDP_CSR[0] & AT91C_UDP_TXCOMP);
   }
}




