#include "common.h"
#include "aes.h"

#define AES		AT91C_BASE_AES
#define AES_ID	AT91C_ID_AES

void aesInit(void)
{
	//enable clock for AES peripheral
	AT91C_BASE_PMC->PMC_PCER = (1<<AES_ID);
	
	//perform software reset on AES peripheral
	AES->AES_CR = AT91C_AES_SWRST;
}

void aesSetKey(uint32_t *aesKey)
{
	for(uint8_t i=0;i<4;i++)
		AES->AES_KEYWxR[i] = aesKey[i];
}



void aesEncrypt(uint32_t *buf, uint16_t len, uint32_t lba)
{
	//configure AES in mode register (CIPHER bit set):
	AES->AES_MR = AT91C_AES_OPMOD_CBC | AT91C_AES_SMOD_PDC | AT91C_AES_CIPHER;
	
	//enter Initialization Vector
	AES->AES_IVxR[0] = 0;
	AES->AES_IVxR[1] = 0;
	AES->AES_IVxR[2] = 0;
	AES->AES_IVxR[3] = lba;
	
	//configure DMA channel
	
	//the same buffer used as receive and transmit buffer
	AES->AES_TPR = (uint32_t) buf;
	AES->AES_RPR = (uint32_t) buf;
	
	//set DMA transfer length
	AES->AES_TCR = len;
	AES->AES_RCR = len;
	
	//start DMA transfer (and encryption too)
	AES->AES_PTCR = AT91C_PDC_TXTEN | AT91C_PDC_RXTEN;
	
	//wait for data to be ready (encrypted)
	while((AES->AES_ISR & AT91C_AES_ENDRX)==0);
	
	//now buf is encrypted!
}

void aesDecrypt(uint32_t *buf, uint16_t len, uint32_t lba)
{
	//configure AES in mode register (CIPHER bit cleared):
	AES->AES_MR = AT91C_AES_OPMOD_CBC | AT91C_AES_SMOD_PDC;
	
	//enter Initialization Vector
	AES->AES_IVxR[0] = 0;
	AES->AES_IVxR[1] = 0;
	AES->AES_IVxR[2] = 0;
	AES->AES_IVxR[3] = lba;
	
	//configure DMA channel
	
	//the same buffer used as receive and transmit buffer
	AES->AES_TPR = (uint32_t) buf;
	AES->AES_RPR = (uint32_t) buf;
	
	//set DMA transfer length
	AES->AES_TCR = len;
	AES->AES_RCR = len;
	
	//start DMA transfer (and decryption too)
	AES->AES_PTCR = AT91C_PDC_TXTEN | AT91C_PDC_RXTEN;
	
	//wait for data to be ready (decrypted)
	while((AES->AES_ISR & AT91C_AES_ENDRX)==0);
	
	//now buf is decrypted!
}

