#include "common.h"
#include "aes.h"

void init(void)
{
	// enable peripheral clock for PIO controllers
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA);
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOB);
	
	// enable reset-key on board 
	AT91C_BASE_RSTC->RSTC_RMR = (0xA5000000 | AT91C_RSTC_URSTEN);
	
	//open debug channel
	debug_init_default();
	
	//initialize LEDs
	LED_R_INIT;
	LED_Y_INIT;
	LED_G_INIT;
	LED_B_INIT;
}


#define CRYPTO_BUFFER_SIZE	4 //in words
#define CRYPTO_BUFFER_SIZE_BYTES	(4*CRYPTO_BUFFER_SIZE)

uint32_t cipherBuf[CRYPTO_BUFFER_SIZE];
uint32_t key[4] = {0,0,0,0};


void dispHelp(void)
{
	debug_msg("actions:");
	debug_msg("h: display help (these messages)");
	debug_msg("w: write a value to each word of the buffer");
	debug_msg("e: encrypt buffer");
	debug_msg("d: decrypt buffer");
	debug_msg("k: set key");
}

int main( void )
{
	init();
	
	dispHelp();
	
	for(;;)
	{
		debug_msg("data in cipherBuf:");
		for(uint8_t i=0;i<CRYPTO_BUFFER_SIZE_BYTES;i++)
		{
			debug8_t(((uint8_t*)cipherBuf)[i]);
			debug_chr(' ');
		}
		debug_msg("current key is:   ");
		for(uint8_t i=0;i<(4*4);i++)
		{
			debug8_t(((uint8_t*)key)[i]);
			debug_chr(' ');
		}
		
		uint8_t chr;
		
		chr = debug_waitkey();
		
		
		switch(chr)
		{
			
			case 'h':
			{
				dispHelp();
				break;
			}
			
			case 'w':
			{
				debug_msg("writing data to each word of cipherBuf");
				for(uint16_t i=0;i<CRYPTO_BUFFER_SIZE;i++)
				{
					debug_msg("enter word "); debug16_t(i+1); debug_txt(" of "); debug16_t(CRYPTO_BUFFER_SIZE); debug_txt(" contents: ");
					uint32_t newContents = debug_input_hex(0);
					cipherBuf[i] = newContents;
				}
				break;
			}
			
			case 'k':
			{
				debug_msg("set new key");
				for(uint16_t i=0;i<4;i++)
				{
					debug_msg("enter word "); debug16_t(i+1); debug_txt(" of "); debug16_t(4); debug_txt(" contents: ");
					uint32_t newContents = debug_input_hex(0);
					key[i] = newContents;
				}
				aesSetKey(key);
				break;
			}
			
			case 'e':
			{
				debug_msg("data encryption with current key");
				aesEncrypt(cipherBuf,CRYPTO_BUFFER_SIZE);
				break;
			}
			
			case 'd':
			{
				debug_msg("data decryption with current key");
				aesDecrypt(cipherBuf,CRYPTO_BUFFER_SIZE);
				break;
			}
		}//switch(chr)
		
	}
	return(0);
}


