#include "Board.h"


void pioIsr(void);



void init(void)
{
	// enable peripheral clock for PIO controllers
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA);
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOB);
	// enable reset-key on board 
	AT91C_BASE_RSTC->RSTC_RMR = (0xA5000000 | AT91C_RSTC_URSTEN);
	
	//initialize LEDs
	LED_R_INIT;
	LED_Y_INIT;
	LED_G_INIT;
	LED_B_INIT;
	
	//initialize interrupt from pushbuttons
	
	//configure AIC
	const uint8_t PIO_IRQ_PRIORITY = 5;
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC , AT91C_ID_PIOA , PIO_IRQ_PRIORITY , AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL , pioIsr);
	
	//enable PIOA interrupt source in AIC
	AT91F_AIC_EnableIt (AT91C_BASE_AIC , KEYBOARD_PIO_ID);
	
	//once AIC is configured, we may enable interrupts on selected pins of PIOA
	KEYBOARD_PIO->PIO_IER = SW1_MASK | SW2_MASK | SW3_MASK | SW4_MASK;
	
	//additionally, it's possible to enable input filters on selected inputs
	KEYBOARD_PIO->PIO_IFER = SW1_MASK | SW2_MASK | SW3_MASK | SW4_MASK;
	
}




int main( void )
{
	init();
	for(;;)
	{
		LED_B_ON;
		delay_ms(250);
		LED_B_OFF;
		delay_ms(250);
	}
	return(0);
}

void pioIsr(void)
{
	//toggle green diode on each interrupt
	LED_G_TOGGLE;
	
	uint32_t status;
	//read status to inquiry which pins caused this irq
	status = AT91C_BASE_PIOA->PIO_ISR;
	
	//check status with masks
	if(status & SW1_MASK)
	{
		LED_R_ON;
	}
	if(status & SW2_MASK)
	{
		LED_R_OFF;
	}
	if(status & SW3_MASK)
	{
		LED_Y_TOGGLE;
	}
	if(status & SW4_MASK)
	{
		LED_R_OFF;
		LED_G_OFF;
		LED_Y_OFF;
	}
}


