#include "common.h"
#include "lcd_2x16.h"
#include "usart_dma.h"

void init(void)
{
	// enable peripheral clock for PIO controllers
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA);
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOB);
	// enable reset-key on board 
	AT91C_BASE_RSTC->RSTC_RMR = (0xA5000000 | AT91C_RSTC_URSTEN);
	
	debug_init_default();
	debug_msg("debug channel initialized");
	
	//initialize LEDs
	LED_R_INIT;
	LED_Y_INIT;
	LED_G_INIT;
	LED_B_INIT;

	lcdInit();
	lcdLocate(0,0);
	lcdText("press:");
	lcdLocate(1,0);
	lcdText("SW1: TX, SW2: RX");
	
	usartDmaInit();
}




int main( void )
{
	init();
	for(;;)
	{
		if(SW1_PRESSED)
		{
			lcdLocate(0,0);
			lcdText("SW1 pressed!    ");
			debug_msg("sw1 pressed!");
			const char * TEXT_TO_SEND = "This text was sent thru DMA channel!";
//			const char * TEXT_TO_SEND = "this";
			usartDmaTransmit((uint8_t*)TEXT_TO_SEND,strlen(TEXT_TO_SEND));
			delay_ms(200);
		}
		if(SW2_PRESSED)
		{
			lcdLocate(1,0);
			lcdText("waiting for text");
			lcdLocate(0,0);
			lcdText("SW2 pressed!    ");
			debug_msg("sw2 pressed!");
			#define RX_BUFFER_SIZE		10
			uint8_t rxBuffer[RX_BUFFER_SIZE];
			usartDmaReceive((uint8_t*)rxBuffer,RX_BUFFER_SIZE);
			
			lcdLocate(0,0);
			lcdText("received text:  ");
			lcdLocate(1,0);
			uint8_t i;
			for(i=0;i<RX_BUFFER_SIZE;i++) lcdChr(rxBuffer[i]); //print RX_BUFFER_SIZE chars on LCD
			for(i=RX_BUFFER_SIZE;i<16;i++) lcdChr(' ');	//fill rest of the line with spaces
			
			debug_msg("received text: ");
			debug_txt_limit((char*)rxBuffer,RX_BUFFER_SIZE);
			delay_ms(400);
			lcdLocate(0,0);
			lcdText("                ");
		}
	}
	return(0);
}

