#include "common.h"
#include "usart_dma.h"

#define USART_BASE			AT91C_BASE_US0
#define USART_ID			AT91C_ID_US0
#define USART_BAUDRATE		115200
#define AIC_BASE			AT91C_BASE_AIC	//AIC base address
#define USART_PIO			AT91C_BASE_PIOA	//PIO controller that muxes USART pins
#define PIO_AB_SEL_REG		PIO_ASR			//select PIO_ASR or PIO_BSR
#define USART_RX_MASK		AT91C_PA0_RXD0	//PIO pin multiplexed with RXD line
#define USART_TX_MASK		AT91C_PA1_TXD0	//PIO pin multiplexed with TXD line


void usartDmaInit(void)
{
	//enable USART clock
	AT91C_BASE_PMC->PMC_PCER = (1<<USART_ID);
	
	//disable PIO control on USART RX/TX pins
	USART_PIO->PIO_PDR = USART_RX_MASK | USART_TX_MASK;
	
	//select peripheral A or B register for USART pins
	USART_PIO->PIO_AB_SEL_REG = USART_RX_MASK | USART_TX_MASK;
	
	//reset and disable transmitter and receiver
	USART_BASE->US_CR = AT91C_US_RSTRX | AT91C_US_RSTTX | AT91C_US_RXDIS | AT91C_US_TXDIS;
	
	//set baud rate
	USART_BASE->US_BRGR = AT91F_US_Baudrate(MCK,USART_BAUDRATE);
	
	//set mode of operation
	USART_BASE->US_MR = AT91C_US_PAR_NONE | AT91C_US_CHRL_8_BITS;
	
	//enable transmitter and receiver
	USART_BASE->US_CR = AT91C_US_RXEN | AT91C_US_TXEN;
}

//transmits "len" chars from "buffer"
void usartDmaTransmit(uint8_t *buffer, uint16_t len)
{
	//print a message
	debug_msg("DMA Tx transfer...");
	
	//write starting memory location
	USART_BASE->US_TPR = (uint32_t) buffer;
	
	//set number of bytes to transmit
	USART_BASE->US_TCR = len;
	
	//start the DMA TX transfer!
	USART_BASE->US_PTCR = AT91C_PDC_TXTEN;
	
	//wait for ENDTX flag to be set
	while( (USART_BASE->US_CSR & AT91C_US_ENDTX)==0 )
	{
		//meanwhile do some silly things...
		LED_R_ON;
		delay_ms(50);
		LED_R_OFF;
		delay_ms(50);
	}
	
}

//stops program execution and waits for "len" chars
//received chars are written to the "buffer"
void usartDmaReceive(uint8_t *buffer, uint16_t len)
{
	//print a message
	debug_msg("DMA Rx transfer...");
	
	//write starting memory location
	USART_BASE->US_RPR = (uint32_t) buffer;
	
	//set number of chars to receive
	USART_BASE->US_RCR = len;
	
	//start the DMA RX transfer!
	USART_BASE->US_PTCR = AT91C_PDC_RXTEN;
	
	//wait for ENDRX flag to be set
	while( (USART_BASE->US_CSR & AT91C_US_ENDRX)==0 )
	{
		//meanwhile do some silly things...
		LED_B_ON;
		delay_ms(50);
		LED_B_OFF;
		delay_ms(50);
	}
	
	debug_txt(" done");
}


