#include "common.h"
#include "spi.h"
#include "sd_spi.h"


void init(void)
{
	// enable peripheral clock for PIO controllers
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA);
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOB);
	
	// enable reset-key on board 
	AT91C_BASE_RSTC->RSTC_RMR = (0xA5000000 | AT91C_RSTC_URSTEN);
	
	//open debug channel
	debug_init_default();
	
	//initialize LEDs
	LED_R_INIT;
	LED_Y_INIT;
	LED_G_INIT;
	LED_B_INIT;
	
	spiInit();
	
	//initialize SD Card
	if(sdCardInit() == OK)
	{
		debug_msg("SD card initialized OK");
	}
	else
	{
		debug_msg("SD card initialization FAILED!");
		while(1);
	}
	
}


#define BLOCK_SIZE			512	//bytes
#define BLOCKS_IN_BUFFER	1
#define BUF_SIZE			( BLOCKS_IN_BUFFER * BLOCK_SIZE )

int main( void )
{
	init();
	
	uint8_t cardBuf[BUF_SIZE];
	
	debug_msg("actions:");
	debug_msg("c: read card capacity");
	debug_msg("r: read a block from card");
	debug_msg("w: write a block to card");
	
	for(;;)
	{
		uint8_t key;
		
		key = debug_inkey();
		
		switch(key)
		{
			case 'c':
			{
				debug_msg("testing sdGetSizeInfo func");
				uint32_t numberOfBlocks = 0;
				uint16_t blockSize = 0;
				sdGetSizeInfo( &numberOfBlocks , &blockSize );
				debug_value("number of blocks: ", numberOfBlocks);
				debug_value("block size: ", blockSize );
				break;
			}//case 'c':
			case 'r':
			{
				uint32_t lba;
				debug_msg("testing sdReadBlock func");
				debug_msg("enter block number in hex: ");
				lba = debug_input_hex(0);
				if( sdReadBlock( lba, cardBuf ) != FAIL)
				{
					debug_msg("sdReadBlock OK, contents:");
					debug_dump( &cardBuf , BUF_SIZE );
				}
				else
				{
					debug_msg("sdReadBlock returned FAIL status - check the card");
				}
				break;
			}//case 'r':
			case 'w':
			{
				uint32_t lba;
				debug_msg("testing sdWriteBlock func");
				debug_msg("SD card contents will be corrupted!!! (press y to continue, any other key to abort)");
				if(debug_waitkey() == 'y')
				{
					debug_msg("enter block number in hex: ");
					lba = debug_input_hex(0);
					uint8_t pattern;
					debug_msg("enter pattern in hex: ");
					pattern = debug_input_hex(0);
					uint16_t i;
					for(i=0; i<BLOCK_SIZE; i++)
					{
						cardBuf[i] = pattern;
					}
					sdWriteBlock( lba , cardBuf );
					debug_msg("done");
				}
				else
				{
					debug_msg("aborted");
				}
				break;
			}//case 'w':
		}//switch(key)
		
	}
	return(0);
}


